// -*- c++ -*-
//
// Software License for MTL
// 
// Copyright (c) 2001-2005 The Trustees of Indiana University. All rights reserved.
// Copyright (c) 1998-2001 University of Notre Dame. All rights reserved.
// Authors: Andrew Lumsdaine, Jeremy G. Siek, Lie-Quan Lee
// 
// This file is part of the Matrix Template Library
// 
// See also license.mtl.txt in the distribution.
//===========================================================================

#include "mtl/utils.h"
#include "mtl/matrix.h"
#include "matrix_test.h"

// matrix_attr.h is generated by make_and_test.pl and defines
// NUMTYPE, SHAPE, STORAGE, ORIEN, and TESTNAME
// you can create your own for testing purposes
#include "matrix_attr.h"

template <class Matrix, class Trans>
bool trans_test(const Matrix& A, const Trans& T, std::string test_name)
{
  for (typename Matrix::const_iterator i = A.begin();
       i != A.end(); ++i)
    for (typename Matrix::OneD::const_iterator j = (*i).begin();
	 j != (*i).end(); ++j)
      if (*j != T(j.column(), j.row())) {
	std::cerr << "**** FAILED: (trans test) " << test_name.c_str() << " ****" << std::endl;
	return false;
      }
  return true;
}


template <class Matrix>
bool trans_test(const Matrix& A, std::string test_name)
{
  if (trans_test(A, mtl::trans(A), test_name) 
      && trans_test(mtl::trans(A), A, test_name)) {
    std::cout << test_name.c_str() << " passed trans test" << std::endl;
    return true;
  } else
    return false;
}

template <class Matrix>
void
do_test(Matrix& A, std::string test_name)
{
  using namespace mtl;

  typedef typename mtl::matrix_traits<Matrix>::value_type T;
  typedef typename mtl::matrix_traits<Matrix>::size_type Int;

  matrix_fill(A);

  trans_test(A, test_name);
}


int
main(int argc, char* argv[])
{
  if (argc < 5) {
    std::cerr << "matrix_test <M> <N> <SUB> <SUPER>" << std::endl;
    return -1;
  }

  using namespace mtl;
  using std::string;

  const int M = atoi(argv[1]);
  const int N = atoi(argv[2]);
  const int SUB = atoi(argv[3]);
  const int SUP = atoi(argv[4]);

  std::cout << "M: " << M << " N: " << N 
       << " SUB: " << SUB << " SUPER: " << SUP << std::endl;

  typedef matrix<NUMTYPE, SHAPE, STORAGE, ORIEN>::type Matrix;

  string test_name = TESTNAME;
  Matrix* a = 0;

  create_and_run(M, N, SUB, SUP, test_name, a, Matrix::shape());

  return 0;
}
